import java.util.List;
import java.util.Random;

/**
 * Ein einfaches Modell eines Hasen.
 * Ein Hase altert, bewegt sich, gebrt Nachwuchs und stirbt.
 * 
 * @author David J. Barnes und Michael Klling
 * @version 2006.03.30
 */
public class Hase extends Tier
{
    // Eigenschaften aller Hasen (statische Datenfelder).

    // Das Alter, in dem ein Hase gebrfhig wird.
    private static final int GEBAER_ALTER = 5;
    // Das Hchstalter eines Hasen.
    private static final int MAX_ALTER = 50;
    // Die Wahrscheinlichkeit, mit der ein Hase Nachwuchs gebrt.
    private static final double GEBAER_WAHRSCHEINLICHKEIT = 0.15;
    // Die maximale Gre eines Wurfes (Anzahl der Jungen)
    private static final int MAX_WURFGROESSE = 5;
    // Ein Zufallsgenerator, der die Geburten beeinflusst.
    private static final Random rand = new Random();
    
    // Individuelle Eigenschaften eines Hasen (Instanzfelder).
    
    /**
     * Erzeuge einen neuen Hasen. Ein Hase kann das Alter 0 
     *(neu geboren) oder ein zuflliges Alter haben.
     * @param zufaelligesAlter soll der Hase ein zuflliges Alter haben?
     */
    public Hase(boolean zufaelligesAlter)
    {
        super();
        if(zufaelligesAlter) {
            setzeAlter(rand.nextInt(MAX_ALTER));
        }
    }
    
    /**
     * Das was ein Hase die meiste Zeit tut - er luft herum.
     * Manchmal gebrt er Nachwuchs und irgendwann stirbt er
     * an Altersschwche.
     * @param aktuellesFeld Das aktuell belegte Feld.
     * @param naechstesFeld Das zu belegende Feld.
     * @param neueTiere Liste, in die neue Hasen einzufgen sind.
     */
    public void agiere(Feld feld, Feld naechstesFeld, List<Tier> neueTiere)
    {
        alterErhoehen();
        if(istLebendig()) {
            int geburten = gebaereNachwuchs();
            for(int b = 0; b < geburten; b++) {
                Hase neuerHase = new Hase(false);
                neueTiere.add(neuerHase);
                neuerHase.setzePosition(naechstesFeld.zufaelligeNachbarposition(gibPosition()));
                naechstesFeld.platziere(neuerHase);
            }
            Position neuePosition = naechstesFeld.freieNachbarposition(gibPosition());
            // nur in das nchste Feld setzen, wenn eine Position frei ist
            if(neuePosition != null) {
                setzePosition(neuePosition);
                naechstesFeld.platziere(this);
            }
            else {
                // weder Bleiben noch Gehen mglich - berpopulation - kein Platz 
                setzeGestorben();
            }
        }
    }
    
    /**
     * Erhhe das Alter dieses Hasen.
     * Dies kann zu seinem Tod fhren.
     */
    private void alterErhoehen()
    {
        setzeAlter(gibAlter()+1);
        if(gibAlter() > MAX_ALTER) {
            setzeGestorben();
        }
    }
    
    /**
     * Gebre Nachwuchs, wenn dieser Hase gebrfhig ist.
     * @return die Anzahl der Neugeborenen (kann Null sein).
     */
    private int gebaereNachwuchs()
    {
        int geburten = 0;
        if(kannGebaeren() && rand.nextDouble() <= GEBAER_WAHRSCHEINLICHKEIT) {
            geburten = rand.nextInt(MAX_WURFGROESSE) + 1;
        }
        return geburten;
    }

    public String toString()
    {
        return "Hase, Alter " + gibAlter();
    }
    /**
     * Ein Hase kann gebren, wenn er das gebrfhige Alter
     * erreicht hat.
     */
    private boolean kannGebaeren()
    {
        return gibAlter() >= GEBAER_ALTER;
    }
}
